@@ -1,9 +1,31 @@
 from typing import Union
-from .global_functions import centile, lms_value_array_for_measurement_for_reference, sds_for_centile, rounded_sds_for_centile, generate_centile
-from .uk_who import select_reference_data_for_uk_who_chart
-from .trisomy_21 import select_reference_data_for_trisomy_21
-from .turner import select_reference_data_for_turner
-from .constants.reference_constants import FEMALE, HEIGHT, UK_WHO, TURNERS, TRISOMY_21, COLE_TWO_THIRDS_SDS_NINE_CENTILES, COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION, THREE_PERCENT_CENTILE_COLLECTION, UK_WHO_REFERENCES
+from .global_functions import centile, sds_for_centile, rounded_sds_for_centile, generate_centile
+from .constants.reference_constants import (
+    CDC_REFERENCES, 
+    CDC,
+    COLE_TWO_THIRDS_SDS_NINE_CENTILES, 
+    COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION,
+    EIGHTY_FIVE_PERCENT_CENTILES,
+    EIGHTY_FIVE_PERCENT_CENTILE_COLLECTION,
+    EXTENDED_WHO_CENTILES,
+    EXTENDED_WHO_CENTILES_COLLECTION,
+    FEMALE,
+    FENTON,
+    FIVE_PERCENT_CENTILES,
+    FIVE_PERCENT_CENTILE_COLLECTION,
+    HEIGHT, 
+    THREE_PERCENT_CENTILES,
+    THREE_PERCENT_CENTILE_COLLECTION,
+    TRISOMY_21, 
+    TRISOMY_21_AAP,
+    TRISOMY_21_AAP_REFERENCES,
+    TURNERS, 
+    UK_WHO, 
+    UK_WHO_REFERENCES, 
+    UK_WHO_CHILD,
+    WHO,
+    WHO_REFERENCES
+)
 
 """
 Public chart functions
@@ -36,78 +58,27 @@ def create_chart(
             sex=sex, 
             centile_format=centile_format, 
             is_sds=is_sds)
+    elif reference == CDC:
+        return create_cdc_chart(
+            measurement_method=measurement_method, 
+            sex=sex, 
+            centile_format=centile_format, 
+            is_sds=is_sds)
+    elif reference == TRISOMY_21_AAP:
+        return create_trisomy_21_aap_chart(
+            measurement_method=measurement_method,
+            sex=sex,
+            centile_format=centile_format,
+            is_sds=is_sds)
+    elif reference == WHO:
+        return create_who_chart(
+            measurement_method=measurement_method,
+            sex=sex,
+            centile_format=centile_format,
+            is_sds=is_sds)
     else:
         print("No reference data returned. Is there a spelling mistake in your reference?")
 
-def generate_custom_sds_line(
-    reference: str, 
-    measurement_method: str, 
-    sex: str, 
-    custom_sds: float):
-    # Public function that returns a custom SDS line
-    # the centile reference data
-    
-    ##
-    # iterate through the 4 references that make up UK-WHO
-    # There will be a list for each one. For the other references ther will be only one list
-    ##
-
-    # all data for a given reference are stored here: this is returned to the user
-    reference_data = []
-
-    custom_centile = centile(custom_sds)
-
-    if reference == UK_WHO:
-        for reference_index, reference in enumerate(UK_WHO_REFERENCES):
-            # the centile reference data
-            lms_array_for_measurement=select_reference_data_for_uk_who_chart(
-                uk_who_reference_name=reference, 
-                measurement_method=measurement_method, 
-                sex=sex)
-            centile_data=[]
-            try:
-                centile_data= build_centile_object(
-                    measurement_method=measurement_method,
-                    sex=sex,
-                    lms_array_for_measurement=lms_array_for_measurement,
-                    z=custom_sds,
-                    centile=custom_centile
-                )
-            except:
-                print("Could not generate centile data.")
-                centile_data=[]
-        # all data can now be tagged by reference_name and added to reference_data
-        reference_data.append({reference: centile_data})
-    else:
-        # get the reference data
-        lms_array_for_measurement=[]
-        try:
-            lms_array_for_measurement=select_reference_lms_data(
-                reference=reference,
-                measurement_method=measurement_method,
-                sex=sex
-            )
-        except:
-            print(f"It was not possible to retrieve {reference} data.")
-            lms_array_for_measurement=[]
-
-        try:
-            centile_data=[]
-            centile_data= build_centile_object(
-                    measurement_method=measurement_method,
-                    sex=sex,
-                    lms_array_for_measurement=lms_array_for_measurement,
-                    z=custom_sds,
-                    centile=custom_centile
-                )
-        except:
-            print("Could not generate sds data.")
-            centile_data=[]
-
-        reference_data.append({reference: centile_data})
-
-    return reference_data
-
     """
     Return object structure
 
@@ -153,20 +124,8 @@ def generate_custom_sds_line(
 """
 private functions
 """
-def select_reference_lms_data(reference: str, measurement_method: str, sex: str)->list:
-    lms_array_for_measurement = []
-    if reference == TURNERS:
-        lms_array_for_measurement=select_reference_data_for_turner(measurement_method=measurement_method, sex=sex)
-    elif reference == TRISOMY_21:
-        lms_array_for_measurement=select_reference_data_for_trisomy_21(measurement_method=measurement_method, sex=sex)
-    else: 
-        raise Exception("No data has been selected!")
-    
-    return lms_array_for_measurement
-    
 
-
-def build_centile_object(measurement_method: str, sex: str, lms_array_for_measurement: list, z: float, centile: float):
+def build_centile_object(reference, measurement_method: str, sex: str, reference_name: str, z: float, centile: float):
     sex_list: dict = {}  # all the data for a given sex are stored here
     measurements: dict = {}  # all the data for a given measurement_method are stored here
     centiles = []  # all generated centiles for a selected centile collection are stored here
@@ -180,8 +139,9 @@ def build_centile_object(measurement_method: str, sex: str, lms_array_for_measur
             centile=centile,
             measurement_method=measurement_method,
             sex=sex,
-            lms_array_for_measurement=lms_array_for_measurement,
-            reference=UK_WHO
+            # lms_array_for_measurement=lms_array_for_measurement,
+            reference_name=reference_name,
+            reference=reference
         )
     except:
         centile_data=None
@@ -217,17 +177,16 @@ def create_uk_who_chart(
     # Alternatively it is possible to pass a custom list of values - if the is_sds flag is False (default) these are centiles
 
     centile_sds_collection = []
-    cole_method = False
+    cole_method = True if centile_format == COLE_TWO_THIRDS_SDS_NINE_CENTILES else False
 
     if (type(centile_format) is list):
+        # a custom list of centiles was provided
         centile_sds_collection = centile_format
-    elif centile_format == COLE_TWO_THIRDS_SDS_NINE_CENTILES:
-        centile_sds_collection = COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
-        cole_method = True
-        is_sds=False
     else:
-        centile_sds_collection = THREE_PERCENT_CENTILE_COLLECTION
+        # a standard centile collection was selected
+        centile_sds_collection = select_centile_format(centile_format)
         is_sds=False
+
     ##
     # iterate through the 4 references that make up UK-WHO
     # There will be a list for each one
@@ -236,29 +195,18 @@ def create_uk_who_chart(
     # all data for a given reference are stored here: this is returned to the user
     reference_data = []
 
-    for reference_index, reference in enumerate(UK_WHO_REFERENCES):
+    for reference_index, reference_name in enumerate(UK_WHO_REFERENCES):
         sex_list: dict = {}  # all the data for a given sex are stored here
         # For each reference we have 2 sexes
-        # for sex_index, sex in enumerate(SEXES):
         # For each sex we have 4 measurement_methods
 
         measurements: dict = {}  # all the data for a given measurement_method are stored here
 
-        # for measurement_index, measurement_method in enumerate(MEASUREMENT_METHODS):
         # for every measurement method we have as many centiles
         # as have been requested
 
         centiles = []  # all generated centiles for a selected centile collection are stored here
 
-        # the centile reference data
-        try:
-            lms_array_for_measurement=select_reference_data_for_uk_who_chart(
-                uk_who_reference_name=reference, 
-                measurement_method=measurement_method, 
-                sex=sex)
-        except:
-            lms_array_for_measurement = []
-
         for centile_index, centile_sds in enumerate(centile_sds_collection):
             # we must create a z for each requested centile
             # if the Cole 9 centiles were selected, these are rounded,
@@ -268,16 +216,15 @@ def create_uk_who_chart(
             z=0.0 #initialise
             centile_value=0.0 #initialise
 
-            if cole_method:
-                z = rounded_sds_for_centile(centile_sds) # a centile was provided, so convert to z
-                centile_value=centile_sds # store the original centile value 
+            if (is_sds):
+                z=centile_sds # an sds was supplied
+                centile_value=centile(centile_sds) # convert the z to a centile and store
             else:
-                if (is_sds):
-                    z=centile_sds # an sds was supplied
-                    centile_value=centile(centile_sds) # convert the z to a centile and store
+                if cole_method:
+                    z = rounded_sds_for_centile(centile_sds)
                 else:
                     z = sds_for_centile(centile_sds) # a centile was provided, so convert to z
-                    centile_value=centile_sds # store the original centile value 
+                centile_value=centile_sds # store the original centile value 
             centile_data = []
 
             try:
@@ -289,7 +236,7 @@ def create_uk_who_chart(
                     centile=centile_value,
                     measurement_method=measurement_method,
                     sex=sex,
-                    lms_array_for_measurement=lms_array_for_measurement,
+                    reference_name=reference_name,
                     reference=UK_WHO,
                     is_sds=is_sds
                 )
@@ -311,7 +258,7 @@ def create_uk_who_chart(
         sex_list.update({sex: measurements})
 
         # all data can now be tagged by reference_name and added to reference_data
-        reference_data.append({reference: sex_list})
+        reference_data.append({reference_name: sex_list})
 
     # returns a list of 4 references, each containing 2 lists for each sex,
     # each sex in turn containing 4 datasets for each measurement_method
@@ -345,18 +292,18 @@ def create_turner_chart(centile_format: Union[str, list], is_sds=False):
     # is different as SDS is rounded to the nearest 2/3
     # Cole method selection is stored in the cole_method flag.
     # If no parameter is passed, default is the Cole method
+    # NOTE: Turner's syndrome only affects girls and reference data only exists for height. This function will only return height data and
+    # relies on error handling elsewhere to catch any other requests for data.
 
     centile_sds_collection = []
     cole_method = False
 
     if (type(centile_format) is list):
+        # a custom list of centiles was provided
         centile_sds_collection = centile_format
-    elif centile_format == COLE_TWO_THIRDS_SDS_NINE_CENTILES:
-        centile_sds_collection = COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
-        cole_method = True
-        is_sds=False
     else:
-        centile_sds_collection = THREE_PERCENT_CENTILE_COLLECTION
+        # a standard centile collection was selected
+        centile_sds_collection = select_centile_format(centile_format)
         is_sds=False
 
     # all data for a the reference are stored here: this is returned to the user
@@ -394,8 +341,8 @@ def create_turner_chart(centile_format: Union[str, list], is_sds=False):
                 z = sds_for_centile(centile_sds)
                 centile_value=centile_sds
         # Collect the LMS values from the correct reference
-        lms_array_for_measurement = select_reference_data_for_turner(
-            measurement_method=HEIGHT, sex=sex)
+        # lms_array_for_measurement = select_reference_data_for_turner(
+        #     measurement_method=HEIGHT, sex=sex)
         # Generate a centile. there will be nine of these if Cole method selected.
         # Some data does not exist at all ages, so any error reflects missing data.
         # If this happens, an empty list is returned.
@@ -405,14 +352,14 @@ def create_turner_chart(centile_format: Union[str, list], is_sds=False):
                 centile=centile_value, 
                 measurement_method=HEIGHT,
                 sex=sex, 
-                lms_array_for_measurement=lms_array_for_measurement, reference=TURNERS,
+                reference_name=TURNERS, reference=TURNERS,
                 is_sds=is_sds)
 
             # Store this centile for a given measurement
             centiles.append({"sds": round(z * 100) / 100,
                             "centile": centile_value, "data": centile_data})
         except Exception as e:
-            print(e)
+            print(f"create_turner chart generate centile error: {e}")
 
     # this is the end of the centile_collection for loop
     # All the centiles for this measurement, sex and reference are added to the measurements list
@@ -444,7 +391,6 @@ def create_turner_chart(centile_format: Union[str, list], is_sds=False):
     }]
     """
 
-
 def create_trisomy_21_chart(measurement_method: str, sex: str, centile_format: Union[str, list], is_sds=False):
    # user selects which centile collection they want
     # If the Cole method is selected, conversion between centile and SDS
@@ -456,28 +402,19 @@ def create_trisomy_21_chart(measurement_method: str, sex: str, centile_format: U
     cole_method = False
 
     if (type(centile_format) is list):
+        # a custom list of centiles was provided
         centile_sds_collection = centile_format
-    elif centile_format == COLE_TWO_THIRDS_SDS_NINE_CENTILES:
-        centile_sds_collection = COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
-        cole_method = True
-        is_sds=False
     else:
-        centile_sds_collection = THREE_PERCENT_CENTILE_COLLECTION
+        # a standard centile collection was selected
+        centile_sds_collection = select_centile_format(centile_format)
         is_sds=False
 
     # all data for a the reference are stored here: this is returned to the user
     reference_data = {}
     sex_list: dict = {}
 
-    # for sex_index, sex in enumerate(SEXES):
-    # For each sex we have 4 measurement_methods
-
     measurements: dict = {}  # all the data for a given measurement_method are stored here
 
-    # for measurement_index, measurement_method in enumerate(MEASUREMENT_METHODS):
-    # for every measurement method we have as many centiles
-    # as have been requested
-
     centiles = []  # all generated centiles for a selected centile collection are stored here
 
     for centile_index, centile_sds in enumerate(centile_sds_collection):
@@ -495,9 +432,7 @@ def create_trisomy_21_chart(measurement_method: str, sex: str, centile_format: U
             else:
                 z = sds_for_centile(centile_sds)
                 centile_value=centile_sds
-        # Collect the LMS values from the correct reference
-        lms_array_for_measurement = select_reference_data_for_trisomy_21(
-            measurement_method=measurement_method, sex=sex)
+
         # Generate a centile. there will be nine of these if Cole method selected.
         # Some data does not exist at all ages, so any error reflects missing data.
         # If this happens, an empty list is returned.
@@ -507,15 +442,15 @@ def create_trisomy_21_chart(measurement_method: str, sex: str, centile_format: U
                 centile=centile_value, 
                 measurement_method=measurement_method,
                 sex=sex, 
-                lms_array_for_measurement=lms_array_for_measurement, 
                 reference=TRISOMY_21,
+                reference_name=TRISOMY_21,
                 is_sds=is_sds)
 
             # Store this centile for a given measurement
             centiles.append({"sds": round(z, 2),
                             "centile": centile_value, "data": centile_data})
         except Exception as e:
-            print(f"{e}")
+            print(f"generate_centile error: {e}")
 
     # this is the end of the centile_collection for loop
     # All the centiles for this measurement, sex and reference are added to the measurements list
@@ -546,3 +481,426 @@ def create_trisomy_21_chart(measurement_method: str, sex: str, centile_format: U
         female {...}
     }]
     """
+
+def create_cdc_chart(
+        measurement_method: str, 
+        sex: str, 
+        centile_format: Union[str, list] = COLE_TWO_THIRDS_SDS_NINE_CENTILES, 
+        is_sds = False
+    ):
+
+    # user selects which centile collection they want, for sex and measurement_method
+    # If the Cole method is selected, conversion between centile and SDS
+    # is different as SDS is rounded to the nearest 2/3
+    # Cole method selection is stored in the cole_method flag.
+    # If no parameter is passed, default is the Cole method
+    # Alternatively it is possible to pass a custom list of values - if the is_sds flag is False (default) these are centiles
+    
+    centile_sds_collection = []
+    cole_method = False
+
+    if (type(centile_format) is list):
+        # a custom list of centiles was provided
+        centile_sds_collection = centile_format
+    else:
+        # a standard centile collection was selected
+        centile_sds_collection = select_centile_format(centile_format)
+        is_sds=False
+
+    ##
+    # iterate through the 3 references that make up CDC (Fenton, WHO, CDC  itself)
+    # There will be a list for each one
+    ##
+
+    # all data for a given reference are stored here: this is returned to the user
+    reference_data = []
+
+    for reference_index, reference_name in enumerate(CDC_REFERENCES):
+        sex_list: dict = {}  # all the data for a given sex are stored here
+        # For each reference we have 2 sexes
+        
+        measurements: dict = {}  # all the data for a given measurement_method are stored here
+
+        centiles = []  # all generated centiles for a selected centile collection are stored here
+
+        for centile_index, centile_sds in enumerate(centile_sds_collection):
+            # we must create a z for each requested centile
+            # if the Cole 9 centiles were selected, these are rounded,
+            # so conversion to SDS is different
+            # Otherwise standard conversation of centile to z is used
+
+            z=0.0 #initialise
+            centile_value=0.0 #initialise
+
+            if cole_method:
+                z = rounded_sds_for_centile(centile_sds) # a centile was provided, so convert to z
+                centile_value=centile_sds # store the original centile value 
+            else:
+                if (is_sds):
+                    z=centile_sds # an sds was supplied
+                    centile_value=centile(centile_sds) # convert the z to a centile and store
+                else:
+                    z = sds_for_centile(centile_sds) # a centile was provided, so convert to z
+                    centile_value=centile_sds # store the original centile value 
+            centile_data = []
+
+            try:
+                # Generate a centile. there will be nine of these if Cole method selected.
+                # Some data does not exist at all ages, so any error reflects missing data.
+                # If this happens, an empty list is returned.
+                centile_data = generate_centile(
+                    z=z,
+                    centile=centile_value,
+                    measurement_method=measurement_method,
+                    sex=sex,
+                    reference=CDC,
+                    reference_name=reference_name,
+                    is_sds=is_sds
+                )
+            except LookupError as e:
+                print(f"Not possible to generate centile data for CDC {measurement_method} in {sex}s. {e}")
+                centile_data=None
+            # Store this centile for a given measurement
+            
+            centiles.append({"sds": round(z * 100) / 100,
+                        "centile": centile_value, "data": centile_data})
+
+        # this is the end of the centile_collection for loop
+        # All the centiles for this measurement, sex and reference are added to the measurements list
+        measurements.update({measurement_method: centiles})
+
+        # this is the end of the measurement_methods loop
+        # All data for all measurement_methods for this sex are added to the sex_list list
+
+        sex_list.update({sex: measurements})
+
+        # all data can now be tagged by reference_name and added to reference_data
+        reference_data.append({reference_name: sex_list})
+
+    # returns a list of 4 references, each containing 2 lists for each sex,
+    # each sex in turn containing 4 datasets for each measurement_method
+    return reference_data
+
+    """
+    # return object structure
+    [ cdc_infant: {
+        male: {
+            height: [
+                {
+                    sds: -2.667,
+                    centile: 0.4
+                    data: [{l: , x: , y: }, ....]
+                }
+            ],
+            weight: [...],
+            bmi: [...],
+            ofc: [...]
+        },
+        female {...}
+    },
+    cdc_child: {
+        male: {
+            height: [
+                {
+                    sds: -2.667,
+                    centile: 0.4
+                    data: [{l: , x: , y: }, ....]
+                }
+            ],
+            weight: [...],
+            bmi: [...],
+            ofc: [...]
+            },
+        female {...}
+        }
+    ]
+    """
+
+def create_trisomy_21_aap_chart(measurement_method: str, sex: str, centile_format: Union[str, list], is_sds=False):
+    # user selects which centile collection they want, for sex and measurement_method
+    # If the Cole method is selected, conversion between centile and SDS
+    # is different as SDS is rounded to the nearest 2/3
+    # Cole method selection is stored in the cole_method flag.
+    # If no parameter is passed, default is the Cole method
+    # Alternatively it is possible to pass a custom list of values - if the is_sds flag is False (default) these are centiles
+    
+    centile_sds_collection = []
+    cole_method = False
+
+    if (type(centile_format) is list):
+        # a custom list of centiles was provided
+        centile_sds_collection = centile_format
+    else:
+        # a standard centile collection was selected
+        centile_sds_collection = select_centile_format(centile_format)
+        is_sds=False
+
+    ##
+    # iterate through the 4 references that make up UK-WHO
+    # There will be a list for each one
+    ##
+
+    # all data for a given reference are stored here: this is returned to the user
+    reference_data = []
+
+    for reference_index, reference_name in enumerate(TRISOMY_21_AAP_REFERENCES):
+        sex_list: dict = {}  # all the data for a given sex are stored here
+        # For each reference we have 2 sexes
+        # For each sex we have 4 measurement_methods
+
+        measurements: dict = {}  # all the data for a given measurement_method are stored here
+
+        centiles = []  # all generated centiles for a selected centile collection are stored here
+
+        for centile_index, centile_sds in enumerate(centile_sds_collection):
+            # we must create a z for each requested centile
+            # if the Cole 9 centiles were selected, these are rounded,
+            # so conversion to SDS is different
+            # Otherwise standard conversation of centile to z is used
+
+            z=0.0 #initialise
+            centile_value=0.0 #initialise
+
+            if cole_method:
+                z = rounded_sds_for_centile(centile_sds) # a centile was provided, so convert to z
+                centile_value=centile_sds # store the original centile value 
+            else:
+                if (is_sds):
+                    z=centile_sds # an sds was supplied
+                    centile_value=centile(centile_sds) # convert the z to a centile and store
+                else:
+                    z = sds_for_centile(centile_sds) # a centile was provided, so convert to z
+                    centile_value=centile_sds # store the original centile value 
+            centile_data = []
+
+            try:
+            # Generate a centile. there will be nine of these if Cole method selected.
+            # Some data does not exist at all ages, so any error reflects missing data.
+            # If this happens, an empty list is returned.
+                centile_data = generate_centile(
+                    z=z,
+                    centile=centile_value,
+                    measurement_method=measurement_method,
+                    sex=sex,
+                    reference=TRISOMY_21_AAP,
+                    reference_name=reference_name,
+                    is_sds=is_sds,
+
+                )
+            except Exception as e:
+                print(f"Not possible to generate centile data for Trisomy 21 (AAP) for {measurement_method} in {sex}s. {e}")
+                centile_data=None
+            # Store this centile for a given measurement
+            
+            centiles.append({"sds": round(z * 100) / 100,
+                        "centile": centile_value, "data": centile_data})
+
+        # this is the end of the centile_collection for loop
+        # All the centiles for this measurement, sex and reference are added to the measurements list
+        measurements.update({measurement_method: centiles})
+
+        # this is the end of the measurement_methods loop
+        # All data for all measurement_methods for this sex are added to the sex_list list
+
+        sex_list.update({sex: measurements})
+
+        # all data can now be tagged by reference_name and added to reference_data
+        reference_data.append({reference_name: sex_list})
+
+    # returns a list of 2 references, each containing 2 lists for each sex,
+    # each sex in turn containing 4 datasets for each measurement_method
+    return reference_data
+
+    """
+    # return object structure
+    [ trisomy_21_aap_infant: {
+        male: {
+            height: [
+                {
+                    sds: -2.667,
+                    centile: 0.4
+                    data: [{l: , x: , y: }, ....]
+                }
+            ],
+            weight: [...],
+            bmi: [...],
+            ofc: [...]
+        },
+        female {...}
+    },
+    trisomy_aap_child: {
+        male: {
+            height: [
+                {
+                    sds: -2.667,
+                    centile: 0.4
+                    data: [{l: , x: , y: }, ....]
+                }
+            ],
+            weight: [...],
+            bmi: [...],
+            ofc: [...]
+            },
+        female {...}
+        }
+    ]
+    """
+
+def create_who_chart(
+        measurement_method: str, 
+        sex: str, 
+        centile_format: Union[str, list] = COLE_TWO_THIRDS_SDS_NINE_CENTILES, 
+        is_sds = False
+    ):
+
+    # user selects which centile collection they want, for sex and measurement_method
+    # If the Cole method is selected, conversion between centile and SDS
+    # is different as SDS is rounded to the nearest 2/3
+    # Cole method selection is stored in the cole_method flag.
+    # If no parameter is passed, default is the Cole method
+    # Alternatively it is possible to pass a custom list of values - if the is_sds flag is False (default) these are centiles
+    
+    centile_sds_collection = []
+    cole_method = False
+
+    if (type(centile_format) is list):
+        # a custom list of centiles was provided
+        centile_sds_collection = centile_format
+    else:
+        # a standard centile collection was selected
+        centile_sds_collection = select_centile_format(centile_format)
+        is_sds=False
+
+    ##
+    # iterate through the 3 references that make up CDC (Fenton, WHO, CDC  itself)
+    # There will be a list for each one
+    ##
+
+    # all data for a given reference are stored here: this is returned to the user
+    reference_data = []
+
+    for reference_index, reference_name in enumerate(WHO_REFERENCES):
+        sex_list: dict = {}  # all the data for a given sex are stored here
+        # For each reference we have 2 sexes
+        # For each sex we have 4 measurement_methods
+
+        measurements: dict = {}  # all the data for a given measurement_method are stored here
+
+        # for every measurement method we have as many centiles
+        # as have been requested
+
+        centiles = []  # all generated centiles for a selected centile collection are stored here
+
+        for centile_index, centile_sds in enumerate(centile_sds_collection):
+            # we must create a z for each requested centile
+            # if the Cole 9 centiles were selected, these are rounded,
+            # so conversion to SDS is different
+            # Otherwise standard conversation of centile to z is used
+
+            z=0.0 #initialise
+            centile_value=0.0 #initialise
+
+            if cole_method:
+                z = rounded_sds_for_centile(centile_sds) # a centile was provided, so convert to z
+                centile_value=centile_sds # store the original centile value 
+            else:
+                if (is_sds):
+                    z=centile_sds # an sds was supplied
+                    centile_value=centile(centile_sds) # convert the z to a centile and store
+                else:
+                    z = sds_for_centile(centile_sds) # a centile was provided, so convert to z
+                    centile_value=centile_sds # store the original centile value 
+            centile_data = []
+
+            try:
+                # Generate a centile. there will be nine of these if Cole method selected.
+                # Some data does not exist at all ages, so any error reflects missing data.
+                # If this happens, an empty list is returned.
+                centile_data = generate_centile(
+                    z=z,
+                    centile=centile_value,
+                    measurement_method=measurement_method,
+                    sex=sex,
+                    reference=WHO,
+                    is_sds=is_sds,
+                    reference_name=reference_name
+                )
+            except LookupError as e:
+                print(f"Not possible to generate centile data for WHO {measurement_method} in {sex}s. {e}")
+                centile_data=None
+            # Store this centile for a given measurement
+            
+            centiles.append({"sds": round(z * 100) / 100,
+                        "centile": centile_value, "data": centile_data})
+
+        # this is the end of the centile_collection for loop
+        # All the centiles for this measurement, sex and reference are added to the measurements list
+        measurements.update({measurement_method: centiles})
+
+        # this is the end of the measurement_methods loop
+        # All data for all measurement_methods for this sex are added to the sex_list list
+
+        sex_list.update({sex: measurements})
+
+        # all data can now be tagged by reference_name and added to reference_data
+        reference_data.append({reference_name: sex_list})
+
+    # returns a list of 4 references, each containing 2 lists for each sex,
+    # each sex in turn containing 4 datasets for each measurement_method
+    return reference_data
+
+    """
+    # return object structure
+    [ cdc_infant: {
+        male: {
+            height: [
+                {
+                    sds: -2.667,
+                    centile: 0.4
+                    data: [{l: , x: , y: }, ....]
+                }
+            ],
+            weight: [...],
+            bmi: [...],
+            ofc: [...]
+        },
+        female {...}
+    },
+    cdc_child: {
+        male: {
+            height: [
+                {
+                    sds: -2.667,
+                    centile: 0.4
+                    data: [{l: , x: , y: }, ....]
+                }
+            ],
+            weight: [...],
+            bmi: [...],
+            ofc: [...]
+            },
+        female {...}
+        }
+    ]
+    """
+        
+
+
+def select_centile_format(centile_format: str):
+    """
+    Select the centile format
+    Helper function to select the correct centile collection
+    the pre-defined collections are in the constants file and have string names: 'cole-nine-centiles', 'three-percent-centiles', 'five-percent-centiles', 'eighty-five-percent_centiles'
+    """
+    if centile_format == COLE_TWO_THIRDS_SDS_NINE_CENTILES:
+        return COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
+    elif centile_format == THREE_PERCENT_CENTILES:
+        return THREE_PERCENT_CENTILE_COLLECTION
+    elif centile_format == FIVE_PERCENT_CENTILES:
+        return FIVE_PERCENT_CENTILE_COLLECTION
+    elif centile_format == EIGHTY_FIVE_PERCENT_CENTILES:
+        return EIGHTY_FIVE_PERCENT_CENTILE_COLLECTION
+    elif centile_format == EXTENDED_WHO_CENTILES:
+        return EXTENDED_WHO_CENTILES_COLLECTION
+    else:
+        return COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
\ No newline at end of file
