@@ -1,6 +1,6 @@
 # standard imports
 from datetime import date
-from pprint import pprint
+from typing import Literal
 
 # rcpch imports
 from .centile_bands import centile_band_for_centile
@@ -41,7 +41,7 @@ class Measurement:
         Additionally there are the following optional parameters:
         `gestation_weeks`: (integer) gestation at birth in weeks.
         `gestation_days`: (integer) supplemental days in addition to gestation_weeks at birth.
-        `reference`: ENUM refering to which reference dataset to use: ['uk-who', 'turners-syndrome', 'trisomy-21'].
+        `reference`: ENUM refering to which reference dataset to use: ['uk-who', 'turners-syndrome', 'trisomy-21', 'trisomy-21-aap', 'cdc', 'who'].
         `height_prediction`: decimal years
         `height_prediction_sds`: SDS for height prediction against reference
         `height_prediction_centile`: centile for height prediction against reference
@@ -72,12 +72,6 @@ class Measurement:
         # self.height_prediction_reference = height_prediction_reference
         self.events_text = events_text
 
-        try:
-            self.__validate_measurement_method(
-                measurement_method=measurement_method, observation_value=observation_value)
-            observation_value_error = None
-        except Exception as err:
-            observation_value_error = f"{err}"
 
         # the ages_object receives birth_data and measurement_dates objects
         self.ages_object = self.__calculate_ages(
@@ -86,7 +80,15 @@ class Measurement:
             observation_date=self.observation_date,
             gestation_weeks=self.gestation_weeks,
             gestation_days=self.gestation_days)
-
+        
+        # validate the measurement method to ensure that the observation value is within the expected range - changed to SDS-based cutoffs - issue #32
+        try:
+            self.__validate_measurement_method(
+                measurement_method=measurement_method, observation_value=observation_value, corrected_decimal_age=self.ages_object['measurement_dates']['corrected_decimal_age'], reference=reference, sex=sex)
+            observation_value_error = None
+        except Exception as err:
+            observation_value_error = f"{err}"
+        
         # the calculate_measurements_object receives the child_observation_value and measurement_calculated_values objects
         self.calculated_measurements_object = self.sds_and_centile_for_measurement_method(
             sex=self.sex,
@@ -236,8 +238,10 @@ class Measurement:
         # calculate sds based on reference, age, measurement, sex and prematurity
 
         if corrected_age is None or chronological_age is None:
-            # there has been an age calculation error. Further calculation impossible
+            # there has been an age calculation error. Further calculation impossible - this may be due to a date error or because CDC reference data is not available in preterm infants
+
             self.return_measurement_object = self.__create_measurement_object(
+                reference=reference,
                 measurement_method=measurement_method,
                 observation_value=observation_value,
                 observation_value_error=observation_value_error,
@@ -253,13 +257,18 @@ class Measurement:
                 chronological_percentage_median_bmi=None
             )
             return self.return_measurement_object
-
-        try:
-            corrected_measurement_sds = sds_for_measurement(reference=reference, age=corrected_age, measurement_method=measurement_method,
-                                                            observation_value=observation_value, sex=sex)
-        except Exception as err:
-            corrected_measurement_error = f"{err}"
+        
+            # CDC data cannot be used for preterm infants who are not yet term. We will not be able to calculate SDS scores and centiles so will return none, but signpost to the user that this is what we are doing.
+        if corrected_age < 0 and reference == "cdc":
+            corrected_measurement_error = "This baby is born premature. CDC data is not available for preterm infants."
             corrected_measurement_sds = None
+        else:
+            try:
+                corrected_measurement_sds = sds_for_measurement(reference=reference, age=corrected_age, measurement_method=measurement_method,
+                                                                observation_value=observation_value, sex=sex)
+            except Exception as err:
+                corrected_measurement_error = f"{err}"
+                corrected_measurement_sds = None
 
         try:
             chronological_measurement_sds = sds_for_measurement(reference=reference, age=chronological_age, measurement_method=measurement_method,
@@ -276,16 +285,23 @@ class Measurement:
             try:
                 chronological_measurement_centile = centile(
                     z_score=chronological_measurement_sds)
-            except TypeError as err:
+            except Exception as err:
                 chronological_measurement_error = "Not possible to calculate centile"
                 chronological_measurement_centile = None
             try:
+                if reference == CDC:
+                    if measurement_method == BMI:
+                        centile_format = EIGHTY_FIVE_PERCENT_CENTILES
+                    else:
+                        centile_format = THREE_PERCENT_CENTILES
+                else:
+                    centile_format = COLE_TWO_THIRDS_SDS_NINE_CENTILES
                 chronological_centile_band = centile_band_for_centile(
                     sds=chronological_measurement_sds, 
                     measurement_method=measurement_method,
-                    centile_format=COLE_TWO_THIRDS_SDS_NINE_CENTILES
+                    centile_format=centile_format
                     )
-            except TypeError as err:
+            except Exception as err:
                 chronological_measurement_error = "Not possible to calculate centile"
                 chronological_centile_band = None
 
@@ -297,20 +313,28 @@ class Measurement:
             try:
                 corrected_measurement_centile = centile(
                     z_score=corrected_measurement_sds)
-            except TypeError as err:
+            except Exception as err:
                 corrected_measurement_error = "Not possible to calculate centile"
                 corrected_measurement_centile = None
 
             try:
+                if reference == CDC:
+                    if measurement_method == BMI:
+                        centile_format = EIGHTY_FIVE_PERCENT_CENTILES
+                    else:
+                        centile_format = THREE_PERCENT_CENTILES
+                else:
+                    centile_format = COLE_TWO_THIRDS_SDS_NINE_CENTILES
                 corrected_centile_band = centile_band_for_centile(
                     sds=corrected_measurement_sds, 
                     measurement_method=measurement_method,
-                    centile_format=COLE_TWO_THIRDS_SDS_NINE_CENTILES
+                    centile_format=centile_format
                 )
-            except TypeError as err:
+            except Exception as err:
                 corrected_measurement_error = "Not possible to calculate centile"
                 corrected_centile_band = None
-        
+
+        # calculate BMI centiles and percentage median BMI
         corrected_percentage_median_bmi = None
         chronological_percentage_median_bmi = None
         if measurement_method == BMI and corrected_age is not None and chronological_age is not None:
@@ -337,6 +361,7 @@ class Measurement:
                 chronological_percentage_median_bmi = None
 
         self.return_measurement_object = self.__create_measurement_object(
+            reference=reference,
             measurement_method=measurement_method,
             observation_value=observation_value,
             observation_value_error=observation_value_error,
@@ -370,6 +395,7 @@ class Measurement:
             gestation_weeks = 40
         # calculate ages from dates and gestational ages at birth
 
+
         try:
             self.corrected_decimal_age = corrected_decimal_age(
                 birth_date=birth_date,
@@ -387,6 +413,11 @@ class Measurement:
         except Exception as err:
             self.chronological_decimal_age = None
             chronological_decimal_age_error = f"{err}"
+        
+        # if reference is CDC or WHO, we must treat >37 week infants as term and we also stop correcting for prematurity at 2 years of age
+        if self.reference == CDC or self.reference == WHO and self.corrected_decimal_age  is not None:
+            if (self.corrected_decimal_age >= 2 and gestation_weeks < 37) or (gestation_weeks >= 37 and gestation_weeks <= 42):
+                self.corrected_decimal_age = self.chronological_decimal_age
 
         if self.corrected_decimal_age is None:
             self._age_comments = None
@@ -399,7 +430,8 @@ class Measurement:
                     chronological_decimal_age=self.chronological_decimal_age,
                     corrected_decimal_age=self.corrected_decimal_age,
                     gestation_weeks=gestation_weeks,
-                    gestation_days=gestation_days)
+                    gestation_days=gestation_days,
+                    reference=self.reference)
             except:
                 self.age_comments = None
                 corrected_decimal_age_error = "Error in comment on prematurity."
@@ -466,17 +498,14 @@ class Measurement:
                 self.estimated_date_delivery_string = None
                 chronological_decimal_age_error = "Estimated date of delivery calculation error."
 
-            # ISSUE: #155 observation date COULD be before estimated date delivery in a preterm
             try:
                 self.corrected_calendar_age = chronological_calendar_age(
                     self.estimated_date_delivery, observation_date)
-            except:
+            except Exception as err:
+                # The EDD is still in the future as this preterm baby is not yet term. The error returned is not useful as the function is expecting a birth date in the past but is being passed an EDD in the future.
+                # It is not really an error, but a limitation of the function. The calendar age really is the same as the corrected gestational age here.
                 self.corrected_calendar_age = None
-                if self.estimated_date_delivery > observation_date:
-                    # ISSUE: #157 if observation date is BEFORE birth date then an error should be raised
-                    chronological_decimal_age_error = "The due date is after the observation date - a calendar age cannot be calculated."
-                else:
-                    chronological_decimal_age_error = "A calendar age cannot be calculated."
+                chronological_decimal_age_error = None
 
             try:
                 self.estimated_date_delivery_string = self.estimated_date_delivery.strftime(
@@ -522,6 +551,7 @@ class Measurement:
 
     def __create_measurement_object(
         self,
+        reference: str,
         measurement_method: str,
         observation_value: float,
         observation_value_error: str,
@@ -549,19 +579,6 @@ class Measurement:
         # Only those calculations relevant to the measurement_method requested populate the final JSON
         # object.
 
-        if corrected_centile_value:
-            if corrected_centile_value > 99 or corrected_centile_value < 1:
-                corrected_centile_value = round(corrected_centile_value, 1)
-            else:
-                corrected_centile_value = int(corrected_centile_value)
-
-        if chronological_centile_value:
-            if chronological_centile_value > 99 or chronological_centile_value < 1:
-                chronological_centile_value = round(
-                    chronological_centile_value, 1)
-            else:
-                chronological_centile_value = int(chronological_centile_value)
-
         measurement_calculated_values = {
             "corrected_sds": corrected_sds_value,
             "corrected_centile": corrected_centile_value,
@@ -589,17 +606,31 @@ class Measurement:
     def __validate_measurement_method(
             self,
             measurement_method: str,
-            observation_value: float):
+            observation_value: float,
+            corrected_decimal_age: float,
+            sex: Literal["male", "female"],
+            reference: Literal['uk-who', 'turners-syndrome', 'trisomy-21', 'trisomy-21-aap', 'cdc', 'who'] = 'uk-who'):
 
-        # Private method which accepts a measurement_method (height, weight, bmi or ofc) and
+        # Private method which accepts a measurement_method (height, weight, bmi or ofc), reference and age as well as observation value
         # and returns True if valid
 
         is_valid = False
 
+        observation_value_z_score = None
+        if observation_value is not None:
+            observation_value_z_score = sds_for_measurement(
+                reference=reference, age=corrected_decimal_age, measurement_method=measurement_method, observation_value=observation_value, sex=sex)
+
         if measurement_method == 'bmi':
             if observation_value is None:
                 raise ValueError(
-                    'Missing observation_value for Body Mass Index. Please pass a Body Mass Index in kilograms per metre squared (kg/m2)')
+                    'Missing observation_value for Body Mass Index. Please pass a Body Mass Index in kilograms per metre squared (kg/m²)')
+            elif observation_value_z_score < MINIMUM_BMI_ERROR_SDS:
+                raise ValueError(
+                    f'The Body Mass Index measurement of {observation_value} kg/m² is below -15 SD and considered to be an error.')
+            elif observation_value_z_score > MAXIMUM_BMI_ERROR_SDS:
+                raise ValueError(
+                    f'The Body Mass Index measurement of {observation_value} kg/m² is above +15 SD and considered to be an error.')
             else:
                 is_valid = True
 
@@ -611,13 +642,12 @@ class Measurement:
                 # most likely metres passed instead of cm.
                 raise ValueError(
                     'Height/length must be passed in cm, not metres')
-            elif observation_value < MINIMUM_LENGTH_CM:
-                # a baby is unlikely to be < 30 cm long - probably a data entry error
+            elif observation_value_z_score < MINIMUM_HEIGHT_WEIGHT_OFC_ERROR_SDS:
                 raise ValueError(
-                    f'The height/length you have entered is very low and likely to be an error. Are you sure you meant a height of {observation_value} centimetres?')
-            elif observation_value > MAXIMUM_HEIGHT_CM:
+                    f'The height/length of {observation_value} cm in a child of {round(corrected_decimal_age, 1)} years is below -8 SD and considered to be an error.')
+            elif observation_value_z_score > MAXIMUM_HEIGHT_WEIGHT_OFC_ERROR_SDS:
                 raise ValueError(
-                    f'The height/length you have entered is very high and likely to be an error. Are you sure you meant a height of {observation_value} centimetres?')
+                    f'The height/length of {observation_value} cm in a child of {round(corrected_decimal_age, 1)} years is above +8 SD and considered to be an error.')
             else:
                 is_valid = True
 
@@ -625,13 +655,13 @@ class Measurement:
             if observation_value is None:
                 raise ValueError(
                     'Missing observation_value for weight. Please pass a weight in kilograms.')
-            elif observation_value < MINIMUM_WEIGHT_KG:
+            elif observation_value_z_score < MINIMUM_HEIGHT_WEIGHT_OFC_ERROR_SDS:
                 raise ValueError(
-                    f'Error. {observation_value} kilograms is very low. Please pass an accurate weight in kilograms')
-            elif observation_value > MAXIMUM_WEIGHT_KG:
+                    f'The weight of {observation_value} kg is below -8 SD in a child of {round(corrected_decimal_age, 1)} years and considered to be an error.')
+            elif observation_value_z_score > MAXIMUM_HEIGHT_WEIGHT_OFC_ERROR_SDS:
                 # it is likely the weight is passed in grams, not kg.
                 raise ValueError(
-                    f'{observation_value} kilograms is very high. Weight must be passed in kilograms.')
+                    f'The weight of {observation_value} kg is above +8 SD in a child of {round(corrected_decimal_age, 1)} years and considered to be an error. Note that the weight should be supplied in kilograms.')
             else:
                 is_valid = True
 
@@ -639,12 +669,12 @@ class Measurement:
             if observation_value is None:
                 raise ValueError(
                     'Missing observation_value for head circumference. Please pass a head circumference in centimetres.')
-            elif observation_value < MINIMUM_OFC_CM:
+            elif observation_value_z_score < MINIMUM_HEIGHT_WEIGHT_OFC_ERROR_SDS:
                 raise ValueError(
-                    f'Please check this value: {observation_value}. A head circumference less than 5 centimetres is likely an error. Please pass an accurate head circumference in centimetres.')
-            elif observation_value > MAXIMUM_OFC_CM:
+                    f'The head circumference of {observation_value} cm in a child of {round(corrected_decimal_age, 1)} years is below -8 SD and considered to be an error.')
+            elif observation_value_z_score > MAXIMUM_HEIGHT_WEIGHT_OFC_ERROR_SDS:
                 raise ValueError(
-                    f'Please check this value: {observation_value}. A head circumference > 150 centimetres is likely an error. Please pass an accurate head circumference in cm.')
+                    f'The head circumference of {observation_value} cm in a child of {round(corrected_decimal_age, 1)} years is above +8 SD and considered to be an error.')
             else:
                 is_valid = True
 
